from typing import Optional


CONN_STRING_PATH = r"SOFTWARE\HRE\Cellario"
JSON_DEFAULT_PATH = r"C:\Program Files\HighRes Biosolutions\Cellario\appsettings.Production.json"


def generate_postgres_conn_string(instance: 'Instance') -> Optional[str]:
    if instance.db_type != "postgres":
        # Why??
        return None

    return f"Server=localhost;Database={instance.user};" + \
        f"Username={instance.user};Password={instance.password};" + \
        "Port=5432;SearchPath=cellario;Application Name=CellarioPostgres"


def generate_oracle_conn_string(instance: 'Instance') -> Optional[str]:
    if instance.db_type != "oracle":
        # Again, why??
        return None

    if instance.version_is_43_or_higher():
        return "Data Source=XE;" + \
            f"User ID={instance.user};" + \
            f"Password={instance.password}"
    else:
        return "Provider=OraOLEDB.Oracle;Data Source=XE;" + \
            f"User ID={instance.user};Password={instance.password}"


def set_conn_string(conn_string: str, db_type: str, is_43_or_higher: bool):
    if is_43_or_higher:
        pass
    else:
        set_conn_string_regkey(conn_string)


def set_conn_string_regkey(conn_string: str):
    try:
        import winreg
    except ImportError:
        print("You may not be running on Windows. Aborting.")
        import sys
        sys.exit(1)

    key = winreg.CreateKey(winreg.HKEY_LOCAL_MACHINE, CONN_STRING_PATH)
    winreg.SetValueEx(key, "ConnectionString", 0, winreg.REG_SZ, conn_string)
    winreg.CloseKey(key)


def set_conn_string_json(conn_string: str,
                         db_type: str,
                         json_path: str = JSON_DEFAULT_PATH):
    import json
    try:
        with open(json_path, 'r') as f:
            app_settings = json.load(f)
    except FileNotFoundError as e:
        # Should we generate a default? Probably not?
        raise e

    # Title case correction smh
    if "postgres" in db_type.lower():
        proper_db_type = "Postgres"
    else:
        proper_db_type = "Oracle"

    app_settings["ConnectionStrings"]["Cellario"] = conn_string
    app_settings["DatabaseType"] = proper_db_type

    with open(json_path, 'w') as f:
        json.dump(app_settings, f)
