import tomllib
import os
from pathlib import Path

from typing import List, Tuple, Optional

import default_toml
from instance import Instance
from interface import start_ui
from settings import MainSettings


def resolve_config_path() -> str:
    script_path = os.getcwd()
    return script_path + "/Cellario_Switcher_DB_List.toml"


def write_default_config_file():
    if Path(resolve_config_path()).is_file():
        return
    with open(resolve_config_path(), "x") as f:
        f.write(default_toml.DEFAULT_TOML_STRING)


def read_config_file() -> Tuple[List[Instance], Optional[MainSettings]]:
    with open(resolve_config_path(), "rb") as f:
        unparsed = tomllib.load(f)

    instances = []
    settings = None
    for name, value in unparsed.items():
        if name == "SETTINGS":
            settings = MainSettings(**value)
            continue
        try:
            new_instance = Instance(name, value)
            instances.append(new_instance)
        except ValueError:
            # Potentially log on bad entries
            pass

    if settings is not None:
        return (instances, settings)
    else:
        return (instances, None)


if __name__ == "__main__":
    write_default_config_file()
    start_ui(*read_config_file())
